import functools

from .helpers import make_payload


class JSONRPC:
    def __init__(self, url):
        """Build

        Parameters
        ----------
        url : the URL of the JSON-RPC endpoint.
        """
        self.url = url
        self.session = None

    def dispatch(self, name=None):
        """Dispatch

        Parameters
        ----------
        name : (optional) the JSON-RPC method name, otherwise
            use the function name

        Examples
        --------

        >>> j = JSONRPC()
        >>> j.session = requests.Session()
        >>> @j.dispatch(name="methodName")
            def method(key: str, amount: int) -> None:
                ...
        >>> # call
        >>> result = method()
        """

        def decorator(func):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                payload = make_payload(name or func.__name__, kwargs.copy())
                response = self.session.post(self.url, json=payload)
                response.raise_for_status()
                return response.json()["result"]

            return wrapper

        return decorator



from setuptools import setup

with open("README.md") as f:
    long_description = f.read()

setup(
    name="fluid-jsonrpc",
    author="Blinktrade",
    version="0.0.1",
    install_requires=["requests"],
    license="BSD",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=["fluid"],
    url="https://github.com/blinktrade/fluid-jsonrpc",
)
